#!/cygdrive/c/Perl/bin/perl.exe -wT
#
## Because cygwin is not happy about DBI >:P
##!/bin/perl -wT
#
###########################################################
# tmdb.pl -
#   Dump one or more TaskMan export files into a database
#
# Version: 1.0
#
# Written by: Justin Zipkin
#             justin.zipkin@db.com
#
# Modified:   16.11.2005 (jz)
#
# Revision Notes:
# 1.0  - Initial (real) Release
#
###########################################################

use strict;
use warnings;
use Getopt::Std;
use DBI;

###########################################################
### FUNCTION PROTOTYPES
###########################################################
sub ReadFile($);
sub TrimArray(@);
sub CreateSQL($@);

###########################################################
### VARIABLE DECLARATIONS
###########################################################
my(%args);
my($line,$file);
my(@files);
my(@exportfile,@trimmedfile,@sqlstatements);

my(%dbinfo)=(
  db_module   => "mysql",
  db_name     => "taskman",
  db_host     => "localhost",
  db_user     => "username",
  db_pass     => "password"
);

our @VALIDSTRINGS=(
  'task\.....\.common\.name',
  'task\.....\.common\.type',
  'task\.....\.common\.schedule',
  'task\.....\.name',
  'task\.....\.type',
  'task\.....\.schedule');

###########################################################
### START CODE
###########################################################

getopts("hp:",\%args);

# Display help
if (exists($args{h})) {
  &DisplayUsage;
  exit(0);
}

my $dbh = DBI->connect("DBI:$dbinfo{db_module}:database=$dbinfo{db_name};host=$dbinfo{db_host}",$dbinfo{db_user},$dbinfo{db_pass}, {RaiseError => 1});
$dbh->do("truncate tasks");

# Process multiple files based on regexp
if (exists($args{p})) {
  # Get a list of all files ending in .export
  opendir(DIR, ".");
  @files = grep(/$args{p}/,readdir(DIR));
  closedir(DIR);
}
else {
  opendir(DIR, ".");
  @files = grep(/\.export$/,readdir(DIR));
  closedir(DIR);
}
  
# For each export file, run through all of the entries
# in the same way as above
foreach $file (@files) {
  @exportfile=ReadFile($file);
  @trimmedfile=TrimArray(@exportfile);
  @sqlstatements=CreateSQL($file, @trimmedfile);
  foreach $line (@sqlstatements) { 
    $dbh->do($line);
    #print $line."\n";
  }
}

###########################################################
# START SUBROUTINES
###########################################################

#### 
# ReadFile($) - Read a file into an array
# 
# Takes: A scalar giving a standard pathname
# Returns: An array containing the file data
###
sub ReadFile($) {
  my(@export);
  open EXPORTFILE, $_[0];
  @export=<EXPORTFILE>;
  close EXPORTFILE;
  return (@export);
}

#### 
# TrimArray(@) - Read a in an array of tasks and trim away
#                unscheduled events.
# 
# Takes: An array of tasks
# Returns: An array of tasks
#
###
sub TrimArray(@) {
  my($line,$grepstring,$joinedstring);
  my($i)=-1;
  my(@trimmedfile,@scheduledtasks);
  my($tasknop,$taskno)=0;
  my(@exportfile)=@_;

  $grepstring=join('|',@VALIDSTRINGS);
  @trimmedfile=grep(/$grepstring/,@exportfile);

  # Determine scheduled tasks by assuming any scheduled task
  # will have more than two fields associated with it.  This
  # is valid because all tasks will have a name and a type.
  # In addition, for a task to run, it must contain scheduling
  # information.
  foreach $line (@trimmedfile) {
    # Get the task number
    $taskno=substr($line,5,4);
  
    # If it's the same as the last one increment i
    if($taskno==$tasknop) {
      $i++;
    }
  
    # If it is a new task nubmer, reset i
    else {
      $i=0;
    }
    
    # If i is equal to 2, this means we have successfully
    # matched i twice since we reset it, therefore this will
    # be our third instance of the task.  This task is
    # scheduled.
    if ($i==2) {
      push(@scheduledtasks,"task." . $taskno . ".");
    }
    $tasknop=$taskno;  
  }

  $joinedstring=join('|',@scheduledtasks);
  @scheduledtasks=grep(/$joinedstring/,@trimmedfile);

  return(@scheduledtasks);
}

#### 
# CreateSQL($@) - Generate SQL code for inserting task
#                 informatino into a MySQL database.
# 
# Takes: A string containing a filename and an array of tasks
# Returns: An array of SQL statements
###
sub CreateSQL($@) {
  my($task,$field,$fieldp,$value,$SQL);

  my @SQLStatements=();
  my $file=shift(@_);
  my @usableinfo=@_;

  my $first=0;
  foreach $line (@usableinfo) {
    $line=~s/\x0D//g;
    $line=~s/\.common\./\./g;
    $task=substr($line,5,4);
    $line=~m/(.*\=)/;
    $field=$1;
    $field=substr($field,10,-1);
    $field=~s/\./_/g;
    $line=~m/(\=.*)/;
    $value=$1;
    $value=substr($value,1);

    if ($field eq "name") {
      if ($first > 0) {
        $SQL=substr($SQL,0,-2);
        push(@SQLStatements,$SQL);
        $SQL='';
      }
      $SQL="INSERT tasks SET `task_id`='$task', `name`='$value', `filename`='$file', ";
      $first++;
    }
    elsif ($field eq "type") {
      $SQL=$SQL."`type`='$value', ";
    }
    elsif (grep(/schedule/,$field)) {
      if (grep(/wait-on_\d+$/,$field)) {
        # Added this to allow for items that do not have a schedule_wait-on.1 value
        # set but have others.  Although this should never be the case, it is on at
        # least one occassion.  Funny enough, in that one case it creates a circular
        # dependency that completely messes up our Gannt chart...
        if (!grep(/wait-on_\d+/,$fieldp)) {
          # If this is not the case (although it always should be) there is something
          # wrong.  Currently this is not matched for JPM transfer so no wait-on field
          # is defined.  This prevents import errors because of its cyclic dependency,
          # but dosen't do a good job expressing there is a problem...
          if(grep(/wait-on_1/,$field)) {
            $SQL=$SQL."`schedule_wait-on`='$value', ";
          }
        }
        else {
          $SQL=substr($SQL,0,-3).', '.$value."', ";
        }
      }
#      if (grep(/wait-on_1$/,$field)) {
#        $SQL=$SQL."`schedule_wait-on`='$value', ";
#      }
#      elsif (grep(/wait-on_/,$field)) {
#        $SQL=substr($SQL,0,-3).', '.$value."', ";
#      }
      else {
        $SQL=$SQL."`$field`='$value', ";
      }
    }
    $fieldp=$field;
  }
  $SQL=substr($SQL,0,-2);
  push(@SQLStatements,$SQL);
  return @SQLStatements;
}

sub DisplayUsage() {
  print "Usage: $0 [-h] | [-p regexp]\n\n";
  print "-h           Display this message\n";
  print "-p [regexp]  A regular expression pattern defining a list of files to process.\n";
  print "             For example $0 -p '_w2k.export\$' will process all files\n";
  print "             ending in _w2k.export\n\n";
  print "Note: If no pattern is supplied '.export\$' will be used as a default\n";
}

###########################################################
# END SUBROUTINES
###########################################################
