<?php
///////////////////////////////////////////////////////////
// calendar.php
///////////////////////////////////////////////////////////
//
// An implementation of the calendar class
//
// Version:    1.0
// Written by: Matthew Piette
//             matthew.piette@db.com
//
// Modified:   
//
///////////////////////////////////////////////////////////

class calendar
{
  private $day, $month, $year, $date;
  
  function __construct($date) 
  {
    $this->date=$date;
    $this->day = date('d', $this->date);
    $this->month = date('m', $this->date);
    $this->year = date('Y', $this->date);
  }
		
	// Used to create a list of links for each month
	// January to December, and print them in a vertical
	// column, 1 per line.
	// Takes: the current date
	// Returns: nothing
	public static function drawMonthLinks($date)
	{
  	//if (!isset($date)) $date=mktime(0,0,0,date('m'),date('d'),date('Y'));
  	$day = date('d', $date);
    $year = date('Y', $date);
  	
    echo "<a href=\"./?date=" . mktime(0,0,0,date('m'),date('d'),date('Y')) . "\">Today</a><br><br>";
    
		for ($counter = 1; $counter <=12; $counter++)
		{
			// Get each month's full name
			$link_name = date('F', mktime(0,0,0,$counter, 1, $year));
			
			// Create the html link for it
			$month_link = "<a href=\"./?date=";
			$month_link .= mktime(0,0,0,$counter,$day,$year);
			$month_link .= "\"> $link_name </a>";
			
			// Output the link to the screen			
			echo "$month_link<br>";
		}
	}
	
	// Used to draw an html calendar
	// Variable conversions (use for inital transform to OO style):
	// num_days_current -->getNumDaysCurr()
	// num_days_last --> getNumDaysPrev()
	// offset --> setOffset()
	// 
	public function drawCalendar ()
	{
		// Get the first day of the month
		$month_start = mktime(0,0,0,$this->month, 1, $this->year);
		
		// Get month name
		$month_name = date('M', $month_start);
		
		// Get which day of the week the month starts on.
		$month_start_day = date('D', $month_start);
		
		switch($month_start_day){
			case "Mon": $offset = 0; break;
			case "Tue": $offset = 1; break;
			case "Wed": $offset = 2; break;
			case "Thu": $offset = 3; break;
			case "Fri": $offset = 4; break;
			case "Sat": $offset = 5; break;
			case "Sun": $offset = 6; break;
		}
							
		// Determine how many days are in the last month.
		if($this->month == 1){
			$num_days_last = cal_days_in_month(0, 12, ($this->year -1));
		} 
		else {
			$num_days_last = cal_days_in_month(0, ($this->month -1), $this->year);
		}
		// Determine how many days are in the current month.
		$num_days_current = cal_days_in_month(0, $this->month, $this->year);
		
		// Build an array for the current days
		// in the month
		for($i = 1; $i <= $num_days_current; $i++){
			$num_days_array[] = $i;
		}
							
		// Build an array for the number of days
		// in last month
		for($i = 1; $i <= $num_days_last; $i++){
			$num_days_last_array[] = $i;
		}
							
		// If the $offset from the starting day of the
		// week happens to be Monday, $offset would be 0, etc
		if($offset > 0){
			$offset_correction = array_slice($num_days_last_array, -$offset, $offset);
			$new_count = array_merge($offset_correction, $num_days_array);
			$offset_count = count($offset_correction);
		}
		// The else statement is to prevent building the $offset array, if not needed.
		else {
			$offset_count = 0;
			$new_count = $num_days_array;
		}
							
		// Count how many days within the two
		// previous arrays merged together
		$current_num = count($new_count);
							
		// There will be 5 HTML table rows
		// each with 7 table data entries 
		// Calculate how many days to append
		// to the end of the final array	
		if($current_num > 35){
			$num_weeks = 6;
			$outset = (42 - $current_num);
		} elseif($current_num < 35){
			$num_weeks = 5;
			$outset = (35 - $current_num);
		}
		if($current_num == 35){
			$num_weeks = 5;
				$outset = 0;
		}
		
		// Outset Correction
		for($i = 1; $i <= $outset; $i++){
			$new_count[] = $i;
		}
							
		// Seperate the $all_days array into weeks. 
		$weeks = array_chunk($new_count, 7);
							
							
		// Build Previous Link
		$previous_link = "<a href=\"".$_SERVER['PHP_SELF']."?date=";
		if($this->month == 1){
			$previous_link .= mktime(0,0,0,12,$this->day,(($this->year) -1));
		} else {
			$previous_link .= mktime(0,0,0,(($this->month) -1),$this->day,$this->year);
		}
		$previous_link .= "\"><< Prev</a>";
				
		// Build the Next Link			
		$next_link = "<a href=\"".$_SERVER['PHP_SELF']."?date=";
		if($this->month == 12){
			$next_link .= mktime(0,0,0,1,$this->day,(($this->year) + 1));
		} else {
			$next_link .= mktime(0,0,0,(($this->month) +1),$this->day,$this->year);
		}
		$next_link .= "\">Next >></a>";
							
							
		// Build the heading portion of the calendar table
		echo "<table align = \"center\" border=\"1\" cellpadding=\"2\" cellspacing=\"0\" class=\"calendar\">\n".
			"<tr>\n".
			"<td colspan=\"7\">\n".
			"<table align=\"center\">\n".
			"<tr>\n".
			"<td colspan=\"2\" width=\"75\" align=\"left\">$previous_link</td>\n".
			"<td class = \"month_name\" colspan=\"3\" width=\"150\" align=\"center\">$month_name $this->year</td>\n".
			"<td colspan=\"2\" width=\"75\" align=\"right\">$next_link</td>\n".
			"</tr>\n".
			"</table>\n".
			"</td>\n".
			"<tr>\n".
			"<th>M</th><th>T</th><th>W</th><th>T</th><th>F</th><th>S</th><th>S</th>\n".
			"</tr>\n";
				
						
		// Break each key of the array into a week and create a new table row 
		// for each week with the days of that week in the table data				
		$i = 0;
		foreach($weeks AS $week){
			echo "<tr>\n";
			foreach($week as $d){
				if($i < $offset_count){
					$day_link = "<a href=\"".$_SERVER['PHP_SELF']."?date=".mktime(0,0,0,$this->month -1,$d,$this->year)."\">$d</a>";
					echo "<td class=\"nonmonthdays\">$day_link</td>\n";
				 }
				if(($i >= $offset_count) && ($i < ($num_weeks * 7) - $outset)){
					$day_link = "<a href=\"".$_SERVER['PHP_SELF']."?date=".mktime(0,0,0,$this->month,$d,$this->year)."\">$d</a>";
					if($this->date == mktime(0,0,0,$this->month,$d,$this->year)){
						echo "<td class=\"today\">$d</td>\n";
					} 
					else {
						echo "<td class=\"days\">$day_link</td>\n";
						}
					} 
					elseif(($outset > 0)) {
						if(($i >= ($num_weeks * 7) - $outset)){
							$day_link = "<a href=\"".$_SERVER['PHP_SELF']."?date=".mktime(0,0,0,$this->month +1,$d,$this->year)."\">$d</a>";
							echo "<td class=\"nonmonthdays\">$day_link</td>\n";
						}
					}
				$i++;
			}
			echo "</tr>\n";   
		}
							
		// Close the table
		echo '<tr><td colspan="7" class="days"> </td></tr>';
		echo '</table>';
		echo '<br />';
	}
}
?>