<?php
///////////////////////////////////////////////////////////
// csv.php.inc
///////////////////////////////////////////////////////////
//
// Script to deliver a CSV representation of the running
// tasks to a web browser
//
// Version:    1.1
// Written by: Justin Zipkin 
//             justin.zipkin@db.com
//
// Modified:   08.11.2005 (jz)
//             15.11.2005 (jz) - extended fileoutput
//
///////////////////////////////////////////////////////////

final class csv extends fileoutput {
  /////////////////////////////////////////////////////////
  // setExtension ()
  //
  // Takes:   Nothing
  // Returns: Nothing
  //
  // Set's the extension of the output file
  //
  /////////////////////////////////////////////////////////  
  protected function setExtension() {
    $this->extension='csv';
  }
  
  /////////////////////////////////////////////////////////
  // setContentType ()
  //
  // Takes:   Nothing
  // Returns: Nothing
  //
  // Set's the content type of the output file
  //
  /////////////////////////////////////////////////////////  
  protected function setContentType() {
    $this->content_type='application/vnd.ms-excel';
  }
  
  /////////////////////////////////////////////////////////
  // createFile ()
  //
  // Takes:   Nothing
  // Returns: A scalar variable populated as a CSV
  //
  // Create a CSV file based off the tasks stored in this
  // csv instance.
  //
  /////////////////////////////////////////////////////////
  protected function createFile() {
    // Prepare the heading row of the CSV data
    $file="Task ID,Filename,Name,Type,Schedule Frequency,Schedule Period,Schedule Start,Schedule Stop,Schedule Wait-on,Schedule Late-alert\n"; 
    
    // Populate the CSV data with tasks
    foreach ($this->tasks as $taskdata) {
      $file.=$taskdata->task_id . ',';
      $file.=$taskdata->filename . ',';
      $file.=$taskdata->name . ',';
      $file.=$taskdata->type . ',';
      $file.=$taskdata->schedule_frequency. ',';
      $file.=$taskdata->schedule_period . ',';
      $file.=$taskdata->schedule_start . ',';
      $file.=$taskdata->schedule_stop . ',';
      // Make sure to replace the commas in the wait_on string with another character
      $file.=preg_replace('/,/',';',$taskdata->schedule_wait_on) . ',';
      $file.=$taskdata->schedule_late_alert . ',';
      $file.="\n";
    }
    
    return $file;
  }
}
