<?php
///////////////////////////////////////////////////////////
// msproject_xml_task.php.inc
///////////////////////////////////////////////////////////
//
// Script to deliver a Microsoft Project representation
// in XML format of the current task and its dependencies
// to a web browser
//
// Version:    1.0
// Written by: Justin Zipkin 
//             justin.zipkin@db.com
//
// Modified:   16.11.2005 (jz)
//
///////////////////////////////////////////////////////////

final class msproject_xml_task extends fileoutput {
  private $task, $taskinfo;
  protected $tasks;
  
  /////////////////////////////////////////////////////////
  // Constructor ($task_id,$filename)
  //
  // Takes:   A task ID and filename
  // Returns: Nothing
  //
  // Populate some variables for use throughout the class
  // and check dependencies
  //
  /////////////////////////////////////////////////////////
  public function __construct($task_id, $filename) {
    if (!class_exists('day')) {
      include 'includes/header.php.inc';
      trigger_error("No day class available.",E_ERROR);
      print "<p class=\"error\">Error: There has been an internal server error, please contact a system administrator if this problem persists.</p>";
      include 'includes/footer.php.inc';
      exit(1);
    }
    
    $this->task=new task($task_id, $filename);
    $this->taskinfo=$this->task->getTask();

    $this->date=mktime(0,0,0,date('m'),date('d'),date('y'));
    
    $this->filename="Taskman_" . $task_id . "_" . preg_replace('/\s/','_',substr($this->taskinfo->name,0,6));
    		
		$this->setContentType();
		$this->setExtension();
  }
  
  /////////////////////////////////////////////////////////
  // setExtension ()
  //
  // Takes:   Nothing
  // Returns: Nothing
  //
  // Set's the extension of the output file
  //
  /////////////////////////////////////////////////////////  
  protected function setExtension() {
    $this->extension='mpp.xml';
  }
  
  /////////////////////////////////////////////////////////
  // setContentType ()
  //
  // Takes:   Nothing
  // Returns: Nothing
  //
  // Set's the content type of the output file
  //
  /////////////////////////////////////////////////////////  
  protected function setContentType() {
    $this->content_type='text/xml';
  }
  
  /////////////////////////////////////////////////////////
  // createFile ()
  //
  // Takes:   Nothing
  // Returns: A scalar variable populated as project XML
  //
  // Create an XML file based off the tasks stored in this
  // msproject_xml instance.
  //
  /////////////////////////////////////////////////////////
  protected function createFile() {
    // Populate an array with the current task, the tasks that it waits on, and the tasks that
    // wait on it
    $this->tasks=$this->task->getDependencies();
    
    // Get the current date in a Microsoft Happy format
    $cur_date=date('Y-m-d\TH:i:s');
    $job_date=date('Y-m-d\TH:i:s',$this->date);
    
    // Prepare our XML file
    $file="<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    $file.="<Project xmlns=\"http://schemas.microsoft.com/project\">";
    $file.="<Name>$this->filename</Name>";
    $file.="<Title>" . $this->task->getTask()->name . "</Title>";
    $file.="<Company>Deutsche Bank</Company>";
    $file.="<Author>Taskman Event Calendar</Author>";
    $file.="<CreationDate>" . $cur_date . "</CreationDate>";
    $file.="<ScheduleFromStart>1</ScheduleFromStart>";
    $file.="<StartDate>" . $job_date . "</StartDate>";
    $file.="<FinishDate>" . $job_date . "</FinishDate>";
    $file.="<WeekStartDay>1</WeekStartDay>";
    $file.="<CurrentDate>" . $cur_date . "</CurrentDate>";
    $file.="<ProjectExternallyEdited>0</ProjectExternallyEdited>";
    $file.="<OutlineCodes/><WBSMasks/><ExtendedAttributes/>";
    $file.="<Calendars><Calendar>";
    $file.="<UID>1</UID>";
    $file.="<Name>Taskman</Name>";
    $file.="<IsBaseCalendar>1</IsBaseCalendar>";
    $file.="<WeekDays>";
    $file.="<WeekDay><DayType>1</DayType><DayWorking>1</DayWorking><WorkingTimes><WorkingTime><FromTime>00:00:00</FromTime><ToTime>23:59:59</ToTime></WorkingTime></WorkingTimes></WeekDay>";
    $file.="<WeekDay><DayType>2</DayType><DayWorking>1</DayWorking><WorkingTimes><WorkingTime><FromTime>00:00:00</FromTime><ToTime>23:59:59</ToTime></WorkingTime></WorkingTimes></WeekDay>";
    $file.="<WeekDay><DayType>3</DayType><DayWorking>1</DayWorking><WorkingTimes><WorkingTime><FromTime>00:00:00</FromTime><ToTime>23:59:59</ToTime></WorkingTime></WorkingTimes></WeekDay>";
    $file.="<WeekDay><DayType>4</DayType><DayWorking>1</DayWorking><WorkingTimes><WorkingTime><FromTime>00:00:00</FromTime><ToTime>23:59:59</ToTime></WorkingTime></WorkingTimes></WeekDay>";
    $file.="<WeekDay><DayType>5</DayType><DayWorking>1</DayWorking><WorkingTimes><WorkingTime><FromTime>00:00:00</FromTime><ToTime>23:59:59</ToTime></WorkingTime></WorkingTimes></WeekDay>";
    $file.="<WeekDay><DayType>6</DayType><DayWorking>1</DayWorking><WorkingTimes><WorkingTime><FromTime>00:00:00</FromTime><ToTime>23:59:59</ToTime></WorkingTime></WorkingTimes></WeekDay>";
    $file.="<WeekDay><DayType>7</DayType><DayWorking>1</DayWorking><WorkingTimes><WorkingTime><FromTime>00:00:00</FromTime><ToTime>23:59:59</ToTime></WorkingTime></WorkingTimes></WeekDay>";
    $file.="</WeekDays>";
    $file.="</Calendar></Calendars>";
    $file.="<Tasks>";

    // Begin populating tasks
    
    // Prepend an empty element to the beginning of an array.  This is because
    // Microsoft Project doesn't like things with an ID of zero? (sometimes?)
    array_unshift($this->tasks,'');
    
    // The ID of each task in Microsoft project will be equal to the ID in the
    // php array.
    for ($n=1; $n<count($this->tasks); $n++) {
      
      // Get the duration of the job in a format of hours, minutes, and seconds
      $start_seconds=(substr($this->tasks[$n]->schedule_start,0,2) * 3600) + (substr($this->tasks[$n]->schedule_start,3,2) * 60) + (substr($this->tasks[$n]->schedule_start,6,2));
      $stop_seconds=(substr($this->tasks[$n]->schedule_stop,0,2) * 3600) + (substr($this->tasks[$n]->schedule_stop,3,2) * 60) + (substr($this->tasks[$n]->schedule_stop,6,2));
      $duration=$stop_seconds - $start_seconds;

      // Get the start and stop times into a format MS Project is okay with
      $start_date=date('Y-m-d',$this->date) . 'T' . $this->tasks[$n]->schedule_start;

      // Hack to make the day one later if the duration is negative
      if ($duration<0) {
        $duration=abs($duration);
        $end_date=date('Y-m-d',$this->date + (24*60*60)) . 'T' . $this->tasks[$n]->schedule_stop;
      }
      else {
        $end_date=date('Y-m-d',$this->date) . 'T' . $this->tasks[$n]->schedule_stop;
      }
      
      $duration_h=floor($duration / 3600);
      $duration_m=floor(($duration - (floor($duration / 3600) * 3600)) / 60);
      $duration_s=($duration % 60);
      
      $file.="<Task>";
      $file.="<UID>$n</UID>";
      $file.="<ID>$n</ID>";
      $file.="<Name>" . $this->parseXML($this->tasks[$n]->name) . "</Name>";
      $file.="<Type>0</Type>";
      $file.="<IsNull>0</IsNull>";
      $file.="<CreateDate>" . $cur_date . "</CreateDate>";
      $file.="<OutlineNumber>$n</OutlineNumber>";
      $file.="<OutlineLevel>1</OutlineLevel>";
      $file.="<Start>$start_date</Start>";
      $file.="<Finish>$end_date</Finish>";
      $file.="<Duration>PT" . $duration_h . "H" . $duration_m . "M" . $duration_s. "S</Duration>";
      $file.="<DurationFormat>5</DurationFormat>";
      $file.="<Recurring>0</Recurring>";
      $file.="<Estimated>0</Estimated>";
      $file.="<Summary>0</Summary>";
      $file.="<Critical>1</Critical>";
      $file.="<ConstraintType>4</ConstraintType>";
      $file.="<CalendarUID>1</CalendarUID>";
      $file.="<HideBar>0</HideBar>";
      
      // Determine the predecessors for each task by exploding their
      // schedule_wait_on field into an array and then iterating through
      // that array.  For each field we also need to iterate through the
      // array of tasks to link up the appropriate ID.
      foreach (explode(", ",$this->tasks[$n]->schedule_wait_on) as $wait_on) {  
        if ($wait_on) {
          for ($i=1; $i<count($this->tasks); $i++) {
            if ($this->tasks[$i]->name==$wait_on) {
              $file.="<PredecessorLink>";
              $file.="<PredecessorUID>$i</PredecessorUID>";
              $file.="<Type>1</Type>";
              $file.="<CrossProject>0</CrossProject>";
              $file.="</PredecessorLink>";
              break;
            }
          }
        }
      }
      $file.="</Task>";
    }
    
    // End populating tasks
    $file.="</Tasks>";
    
    // Finish the project, there are no resources or assignments
    $file.="</Project>";

    return $file;
  }
}
