<?php
///////////////////////////////////////////////////////////
// xml.php.inc
///////////////////////////////////////////////////////////
//
// Script to deliver an XML representation of the running
// tasks to a web browser
//
// Version:    1.1
// Written by: Justin Zipkin 
//             justin.zipkin@db.com
//
// Modified:   08.11.2005 (jz)
//             11.11.2005 (mp)
//             15.11.2005 (jz) - extended fileoutput
//
///////////////////////////////////////////////////////////

final class xml extends fileoutput {
  /////////////////////////////////////////////////////////
  // setExtension ()
  //
  // Takes:   Nothing
  // Returns: Nothing
  //
  // Set's the extension of the output file
  //
  /////////////////////////////////////////////////////////  
  protected function setExtension() {
    $this->extension='xml';
  }
  
  /////////////////////////////////////////////////////////
  // setContentType ()
  //
  // Takes:   Nothing
  // Returns: Nothing
  //
  // Set's the content type of the output file
  //
  /////////////////////////////////////////////////////////  
  protected function setContentType() {
    $this->content_type='text/xml';
  }
  
  /////////////////////////////////////////////////////////
  // createFile ()
  //
  // Takes:   Nothing
  // Returns: A scalar variable populated as an XML
  //
  // Create an XML file based off the tasks stored in this
  // xml instance.
  //
  /////////////////////////////////////////////////////////
  protected function createFile() {
    $file="<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    $file.="<!DOCTYPE tasklist [\n";
    $file.="<!ELEMENT tasklist (task+)>\n";
    $file.="<!ELEMENT task (name,type,schedule)>\n";
    $file.="<!ELEMENT name (#PCDATA)>\n";
    $file.="<!ELEMENT type (#PCDATA)>\n";
    $file.="<!ELEMENT schedule (frequency,period,start,stop,wait-on*,late-alert?)>\n";
    $file.="<!ELEMENT frequency (#PCDATA)>\n";
    $file.="<!ELEMENT period (#PCDATA)>\n";
    $file.="<!ELEMENT start (#PCDATA)>\n";
    $file.="<!ELEMENT stop (#PCDATA)>\n";
    $file.="<!ELEMENT wait-on (#PCDATA)>\n";
    $file.="<!ELEMENT late-alert (#PCDATA)>\n\n";
    $file.="<!ATTLIST tasklist date CDATA #REQUIRED>\n";
    $file.="<!ATTLIST task id CDATA #REQUIRED filename CDATA #REQUIRED>\n";
    $file.="]>\n";

    // Prepare our root element
    $file.="<tasklist date=\"$this->day/$this->month/$this->year\">\n";
		
    foreach ($this->tasks as $taskdata) 
    {
      $file .= "<task id=\"$taskdata->task_id\" filename=\"$taskdata->filename\">\n";
      $file .= "<name>"      . $this->parseXML($taskdata->name)        . "</name>\n";
      $file .= "<type>"      . $this->parseXML($taskdata->type)        . "</type>\n";
      $file .= "<schedule>\n";
      $file .= "<frequency>" . $this->parseXML($taskdata->schedule_frequency) . "</frequency>\n";
      $file .= "<period>"    . $this->parseXML($taskdata->schedule_period)    . "</period>\n";
      $file .= "<start>"     . $this->parseXML($taskdata->schedule_start)     . "</start>\n";
      $file .= "<stop>"      . $this->parseXML($taskdata->schedule_stop)      . "</stop>\n";
      $wait_on=explode(',',$taskdata->schedule_wait_on);
      
      foreach($wait_on as $wait) 
      {
        if ($wait!='') $file .= "<wait-on>"   . $this->parseXML($wait)  . "</wait-on>\n";
      }
      if ($taskdata->schedule_late_alert!='')
        $file .= "<late-alert>" . $this->parseXML($taskdata->schedule_late_alert) . "</late-alert>\n";
      
      $file .= "</schedule>\n";
      $file .= "</task>\n";
    }
    
    // Close our root element
    $file.="</tasklist>\n";
    
    return $file;
  }
}
